/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.role.util;

import it.tidalwave.util.RoleFactory;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProvider;
import lombok.RequiredArgsConstructor;

/***********************************************************************************************************************
 *
 * A dynamic role register that provides a fluent interface approach for programmatically binding Roles to Data.
 * As a result, it provides a {@link Lookup} with the needed {@link CapabilitiesProvider} instances.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class RoleRegister implements Lookup.Provider
  {
    private static final Logger log = LoggerFactory.getLogger(RoleRegister.class);
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @RequiredArgsConstructor
    public class RoleRegistrationBuilder
      {
        @Nonnull
        private final Object role;
        
        /***************************************************************************************************************
         *
         * Registers the Role to the given class.
         * 
         * @param  clazz   the class to register the Role to
         *
         **************************************************************************************************************/
        public void forClass (final @Nonnull Class<?> clazz)
          {
            List<Object> roles = rolesMapByClass.get(clazz);
            
            if (roles == null)
              {
                roles = new ArrayList<Object>();
                rolesMapByClass.put(clazz, roles);
              }
            
            roles.add(role);
          }
        
        /***************************************************************************************************************
         *
         * Registers the Role to objects satisfying the given predicate.
         * FIXME: this is currently limited to subclassses of PresentationModel
         * 
         * @param  predicate  the predicate
         *
         **************************************************************************************************************/
        public void forObjectsThat (final @Nonnull Predicate predicate)
          {
            rolesMapByCondition.put(new Predicate()
              {
                public boolean apply (final @Nonnull Object object) 
                  {
                    return (object instanceof PresentationModel) && predicate.apply(object);
                  }
              }, role);
          }
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @RequiredArgsConstructor
    public class RoleFactoryRegistrationBuilder
      {
        @Nonnull
        private final RoleFactory<?> roleFactory;
        
        /***************************************************************************************************************
         *
         * Registers the Role to the given class.
         * 
         * @param  clazz   the class to register the Role to
         *
         **************************************************************************************************************/
        public void forClass (final @Nonnull Class<?> clazz)
          {
            List<RoleFactory<?>> roleFactories = roleFactoriesMapByClass.get(clazz);
            
            if (roleFactories == null)
              {
                roleFactories = new ArrayList<RoleFactory<?>>();
                roleFactoriesMapByClass.put(clazz, roleFactories);
              }
            
            roleFactories.add(roleFactory);
          }
        
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static interface Predicate
      {
        public boolean apply (@Nonnull Object object);  
      }

    private final Map<Class<?>, List<Object>> rolesMapByClass = new HashMap<Class<?>, List<Object>>();
    
    private final Map<Class<?>, List<RoleFactory<?>>> roleFactoriesMapByClass = new HashMap<Class<?>, List<RoleFactory<?>>>();
    
    private final Map<Predicate, Object> rolesMapByCondition = new HashMap<Predicate, Object>();
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Lookup getLookup() 
      {
        log.debug("getLookup()");
        
        final List<CapabilitiesProvider> capabilitiesProviders = new ArrayList<CapabilitiesProvider>();
        
        for (final Entry<Class<?>, List<Object>> entry : rolesMapByClass.entrySet())
          {
            final Class<?> datumClass = entry.getKey();
            final List<Object> roles = entry.getValue();            
            log.debug(">>>> for {} roles are {}", datumClass, roles);
            capabilitiesProviders.add(new RolesCapabilitiesProvider(datumClass, roles));
          }
                
        for (final Entry<Class<?>, List<RoleFactory<?>>> entry : roleFactoriesMapByClass.entrySet())
          {
            final Class<?> datumClass = entry.getKey();
            final List<RoleFactory<?>> roleFactories = entry.getValue();            
            log.debug(">>>> for {} roleFactories are {}", datumClass, roleFactories);
            capabilitiesProviders.add(new RoleFactoriesCapabilitiesProvider(datumClass, roleFactories));
          }
                
        for (final Entry<Predicate, Object> entry : rolesMapByCondition.entrySet())
          {
            final Predicate predicate = entry.getKey();
            final Object role = entry.getValue();            
            log.debug(">>>> for {} role is {}", predicate, role);
            capabilitiesProviders.add(new PredicateCapabilityProvider<PresentationModel>(PresentationModel.class, role, predicate));
          }
        
        return Lookups.fixed(capabilitiesProviders.toArray());
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public RoleRegistrationBuilder registerRole (final @Nonnull Object role)
      {
        return new RoleRegistrationBuilder(role);
      }    
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public RoleFactoryRegistrationBuilder registerRoleFactory (final @Nonnull RoleFactory<?> roleFactory)
      {
        return new RoleFactoryRegistrationBuilder(roleFactory);
      }    
  }
